@@ -192,110 +192,7 @@ print_x509_info (gnutls_session_t session, int flag, int print_cert)
       }
 }
 
-/* returns true or false, depending on whether the hostname
- * matches to certificate */
-static int
-verify_x509_hostname (gnutls_session_t session, const char *hostname)
-{
-  gnutls_x509_crt_t crt;
-  const gnutls_datum_t *cert_list;
-  unsigned int cert_list_size = 0;
-  int ret;
-
-  cert_list = gnutls_certificate_get_peers (session, &cert_list_size);
-  if (cert_list_size == 0)
-    {
-      fprintf (stderr, "No certificates found!\n");
-      return 0;
-    }
-
-  gnutls_x509_crt_init (&crt);
-  ret =
-      gnutls_x509_crt_import (crt, &cert_list[0],
-                              GNUTLS_X509_FMT_DER);
-  if (ret < 0)
-    {
-      fprintf (stderr, "Decoding error: %s\n",
-               gnutls_strerror (ret));
-      return 0;
-    }
-
-  /* Check the hostname of the first certificate if it matches
-   * the name of the host we connected to.
-   */
-  if (hostname != NULL)
-    {
-      if (gnutls_x509_crt_check_hostname (crt, hostname) == 0)
-        {
-          printf
-             ("- The hostname in the certificate does NOT match '%s'\n",
-              hostname);
-          ret = 0;
-        }
-      else
-        {
-          printf ("- The hostname in the certificate matches '%s'.\n",
-                  hostname);
-          ret = 1;
-        }
-    }
-
-  gnutls_x509_crt_deinit (crt);
-
-  return ret;
-}
-
 #ifdef ENABLE_OPENPGP
-/* returns true or false, depending on whether the hostname
- * matches to certificate */
-static int
-verify_openpgp_hostname (gnutls_session_t session, const char *hostname)
-{
-  gnutls_openpgp_crt_t crt;
-  const gnutls_datum_t *cert_list;
-  unsigned int cert_list_size = 0;
-  int ret;
-
-  cert_list = gnutls_certificate_get_peers (session, &cert_list_size);
-  if (cert_list_size == 0)
-    {
-      fprintf (stderr, "No certificates found!\n");
-      return 0;
-    }
-
-  gnutls_openpgp_crt_init (&crt);
-  ret =
-      gnutls_openpgp_crt_import (crt, &cert_list[0],
-                              GNUTLS_OPENPGP_FMT_RAW);
-  if (ret < 0)
-    {
-      fprintf (stderr, "Decoding error: %s\n",
-               gnutls_strerror (ret));
-      return 0;
-    }
-
-  /* Check the hostname of the first certificate if it matches
-   * the name of the host we connected to.
-   */
-  if (gnutls_openpgp_crt_check_hostname (crt, hostname) == 0)
-    {
-      printf
-             ("- The hostname in the certificate does NOT match '%s'\n",
-              hostname);
-      ret = 0;
-    }
-  else
-    {
-      printf ("- The hostname in the certificate matches '%s'.\n",
-              hostname);
-      ret = 1;
-    }
-
-  gnutls_openpgp_crt_deinit (crt);
-
-  return ret;
-}
-
 static void
 print_openpgp_info_compact (gnutls_session_t session)
 {
@@ -417,9 +314,10 @@ cert_verify (gnutls_session_t session, const char* hostname)
 {
     int rc;
     unsigned int status = 0;
+    gnutls_datum_t out;
     int type;
 
-    rc = gnutls_certificate_verify_peers2 (session, &status);
+    rc = gnutls_certificate_verify_peers3 (session, hostname, &status);
     if (rc == GNUTLS_E_NO_CERTIFICATE_FOUND)
       {
           printf ("- Peer did not send any certificate.\n");
@@ -434,49 +332,17 @@ cert_verify (gnutls_session_t session, const char* hostname)
       }
 
     type = gnutls_certificate_type_get (session);
-    if (type == GNUTLS_CRT_X509)
+    rc = gnutls_certificate_verification_status_print( status, type, &out, 0);
+    if (rc < 0)
       {
-
-          if (status & GNUTLS_CERT_REVOKED)
-              printf ("- Peer's certificate chain revoked\n");
-          if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
-              printf ("- Peer's certificate issuer is unknown\n");
-          if (status & GNUTLS_CERT_SIGNER_NOT_CA)
-              printf ("- Peer's certificate issuer is not a CA\n");
-          if (status & GNUTLS_CERT_INSECURE_ALGORITHM)
-              printf
-                  ("- Peer's certificate chain uses insecure algorithm\n");
-          if (status & GNUTLS_CERT_NOT_ACTIVATED)
-              printf
-                  ("- Peer's certificate chain uses not yet valid certificate\n");
-          if (status & GNUTLS_CERT_EXPIRED)
-              printf
-                  ("- Peer's certificate chain uses expired certificate\n");
-          if (status & GNUTLS_CERT_INVALID)
-              printf ("- Peer's certificate is NOT trusted\n");
-          else
-              printf ("- Peer's certificate is trusted\n");
-
-          rc = verify_x509_hostname (session, hostname);
-          if (rc == 0) status |= GNUTLS_CERT_INVALID;
+          printf ("- Could not print verification flags (err: %s)\n",
+                  gnutls_strerror (rc));
+          return 0;
       }
-    else if (type == GNUTLS_CRT_OPENPGP)
-      {
-          if (status & GNUTLS_CERT_INVALID)
-              printf ("- Peer's key is invalid\n");
-          else
-              printf ("- Peer's key is valid\n");
-          if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
-              printf ("- Could not find a signer of the peer's key\n");
 
-          rc = verify_openpgp_hostname (session, hostname);
-          if (rc == 0) status |= GNUTLS_CERT_INVALID;
-      }
-    else
-      {
-        fprintf(stderr, "Unknown certificate type\n");
-        status |= GNUTLS_CERT_INVALID;
-      }
+    printf ("- Status: %s\n", out.data);
+    
+    gnutls_free(out.data);
 
     if (status)
       return 0;
@@ -590,6 +456,13 @@ print_info (gnutls_session_t session, int verbose, int print_cert)
     gnutls_kx_algorithm_t kx;
     unsigned char session_id[33];
     size_t session_id_size = sizeof (session_id);
+    gnutls_srtp_profile_t srtp_profile;
+    char *desc;
+    int rc;
+
+    desc = gnutls_session_get_desc(session);
+    printf ("- Description: %s\n", desc);
+    gnutls_free(desc);
 
     /* print session ID */
     gnutls_session_get_id (session, session_id, &session_id_size);
@@ -685,10 +558,15 @@ print_info (gnutls_session_t session, int verbose, int print_cert)
             (gnutls_compression_get (session)));
     printf ("- Compression: %s\n", tmp);
 
+#ifdef ENABLE_DTLS_SRTP
+    rc = gnutls_srtp_get_selected_profile (session, &srtp_profile);
+    if (rc == 0)
+      printf ("- SRTP profile: %s\n", gnutls_srtp_get_profile_name (srtp_profile));
+#endif
+
     if (verbose)
       {
           gnutls_datum_t cb;
-          int rc;
 
           rc = gnutls_session_channel_binding (session,
                                                GNUTLS_CB_TLS_UNIQUE, &cb);
@@ -1111,7 +989,7 @@ pin_callback (void *user, int attempt, const char *token_url,
     
   if (cache > 0 && cached_url != NULL)
     {
-      if (strcmp (cached_url, token_url) == 0)
+      if (token_url != NULL && strcmp (cached_url, token_url) == 0)
         {
           if (strlen(pin) >= sizeof(cached_pin))
             {
@@ -1143,7 +1021,11 @@ pin_callback (void *user, int attempt, const char *token_url,
   /* cache */
   strcpy (cached_pin, pin);
   free (cached_url);
-  cached_url = strdup (token_url);
+  if (token_url)
+    cached_url = strdup (token_url);
+  else
+    cached_url = NULL;
+
   cache = MAX_CACHE_TRIES;
 
   return 0;
