@@ -55,7 +55,7 @@
 
 static void cmd_parser (int argc, char **argv);
 static void dane_info(const char* host, const char* proto, unsigned int port, 
-                      unsigned int ca, unsigned int local, common_info_st * cinfo);
+                      unsigned int ca, unsigned int domain, common_info_st * cinfo);
 
 static void dane_check(const char* host, const char* proto, unsigned int port,
                        common_info_st * cinfo);
@@ -166,7 +166,7 @@ cmd_parser (int argc, char **argv)
 
   if (HAVE_OPT(TLSA_RR))
     dane_info (OPT_ARG(HOST), proto, port,
-               HAVE_OPT(CA), HAVE_OPT(LOCAL), &cinfo);
+               HAVE_OPT(CA), ENABLED_OPT(DOMAIN), &cinfo);
   else if (HAVE_OPT(CHECK))
     dane_check (OPT_ARG(CHECK), proto, port, 
                 &cinfo);
@@ -187,12 +187,13 @@ static void dane_check(const char* host, const char* proto, unsigned int port,
 #ifdef HAVE_DANE
 dane_state_t s;
 dane_query_t q;
-int ret;
+int ret, retcode = 0;
 unsigned entries;
 unsigned int flags = DANE_F_IGNORE_LOCAL_RESOLVER, i;
 unsigned int usage, type, match;
 gnutls_datum_t data, file;
 size_t size;
+unsigned vflags = DANE_VFLAG_FAIL_IF_NOT_CHECKED;
 
   if (ENABLED_OPT(LOCAL_DNS))
     flags = 0;
@@ -200,6 +201,12 @@ size_t size;
   if (HAVE_OPT(INSECURE))
     flags |= DANE_F_INSECURE;
 
+  if (HAVE_OPT(CHECK_EE))
+    vflags |= DANE_VFLAG_ONLY_CHECK_EE_USAGE;
+
+  if (HAVE_OPT(CHECK_CA))
+    vflags |= DANE_VFLAG_ONLY_CHECK_CA_USAGE;
+
   printf("Querying %s (%s:%d)...\n", host, proto, port);
   ret = dane_state_init(&s, flags);
   if (ret < 0)
@@ -265,7 +272,7 @@ size_t size;
                 }
 
               ret = dane_verify_crt( s, certs, clist_size, GNUTLS_CRT_X509, 
-                                     host, proto, port, 0, 0, &status);
+                                     host, proto, port, 0, vflags, &status);
               if (ret < 0)
                 error (EXIT_FAILURE, 0, "dane_verify_crt: %s", dane_strerror (ret));
                 
@@ -275,6 +282,8 @@ size_t size;
               
               printf("\nVerification: %s\n", out.data);
               gnutls_free(out.data);
+              
+              if (status != 0) retcode = 1;
 
               for (i=0;i<clist_size;i++)
                 {
@@ -289,6 +298,8 @@ size_t size;
 
   dane_query_deinit(q);
   dane_state_deinit(s);
+  
+  exit(retcode);
 #else
   fprintf(stderr, "This functionality was disabled (GnuTLS was not compiled with support for DANE).\n");
   return;
@@ -296,7 +307,7 @@ size_t size;
 }
 
 static void dane_info(const char* host, const char* proto, unsigned int port, 
-                      unsigned int ca, unsigned int local, common_info_st * cinfo)
+                      unsigned int ca, unsigned int domain, common_info_st * cinfo)
 {
   gnutls_pubkey_t pubkey;
   gnutls_x509_crt_t crt;
@@ -379,7 +390,7 @@ static void dane_info(const char* host, const char* proto, unsigned int port,
   else type = 2;
 
   /* DANE certificate classification crap */
-  if (local==0)
+  if (domain==0)
     {  
       if (ca) usage = 0;
       else usage = 1;
